import 'dart:math';

class Chance {
  // Maximum integer that JS supports.
  static const _maxInt = 9007199254740992;
  static const _minInt = -_maxInt;

  Random _random = new Random();

  /// Returns a random integer.
  ///
  /// Leave out [min] to get a value between 0 and [max].
  /// Leave out all arguments to get a value between [_minInt] and [_maxInt].
  int integer({int min, int max}) {
    // No arguments provided
    if (min == null && max == null) {
      min = _minInt;
      max = _maxInt;

    // Only max provided
    } else if (min == null && max != null)
      min = 0;

    // Only min provided
    else if (min != null && max == null)
      throw new StateError('Must not provide min without max');

    else if (min > max)
      throw new RangeError('Min must not be smaller than max.');

    return (_randomDouble * (max - min + 1) + min).floor();
  }

  /// Returns a random floating / double number.
  ///
  /// Leave out [min] to get a value between 0 and [max].
  /// Leave out all arguments to get a value between 0 and 1.
  double floating({double min, double max}) {
    // No arguments provided
    if (min == null && max == null) {
      min = 0.0;
      max = 1.0;

    // Only max provided
    } else if (min == null && max != null)
      min = 0.0;

    // Only min provided
    else if (min != null && max == null)
      throw new StateError('Must not provide min without max');

    else if (min > max)
      throw new RangeError('Min must not be smaller than max.');

    return _randomDouble * (max - min) + min;
  }

  double get _randomDouble => _random.nextDouble();
}
